% Mosek wrapper to solve IRP partitioning (i.e. cut) problem
% For efficiency, the dual problem (the max flow problem) is solved.
% NOTE: the current implementation requires a Mosek license
% Compute:
% maximize      sum_i(s_i+t_i)
% subject to 
%               sum_{j:(i,j) in I}(lambda_ij)-sum_{j:(j,i) in I}(lambda_ji)-s_i+t_i=c_i
%               s,t,lambda>=0
%%%%%%%%%%%%
% Inputs:  
% Aineq:        n x n matrix s.t. A(i,j)==1 if y(i)<=y(j) and A(i,j)==0 otherwise
% beineq:       n x 1 input vector to isotonic regression, e.g. observations
% m:            same options as to GIRP(.) function
% n:            indices of observations in the group to be cut (for use when there are weights)
%%%%%%%%%%%
% Outputs:  
% x:             solution to partitioning problem
%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%
function [x] = IRP_LP_subproblem_network_dual_mosek(Aineq,bineq,m,n)

clear prob;

prob.c = sparse([ones(2*n,1);zeros(m,1)]);  % coefficients for objective (only s and t have ones, all others zero)
prob.a = Aineq;      
prob.blc  = bineq;  % since we have equalities in the network, we set the upper and lower bounds equal
prob.buc  = bineq;
prob.blx  = zeros(m+2*n,1);   % all variables have lower bounds of zero
prob.bux = [];

param = [];
% settings for using network simplex
param.MSK_IPAR_SIM_NETWORK_DETECT=0;
param.MSK_IPAR_OPTIMIZER='MSK_OPTIMIZER_FREE_SIMPLEX';

% Perform the optimization.
[r,res] = mosekopt('minimize echo(0)',prob,param); 
clear('prob','Aineq');
if strcmp(res.rcodestr,'MSK_RES_OK')==0
    disp('Error in MOSEK network solver');
end

x=round(res.sol.bas.y); % return the dual solution to the network problem
ind=find(x==0);
if ~isempty(ind) % DEBUG IF there is x(i)=0
    if sum(sign(x)==1)==(n-length(ind))
        x(ind)=1;
    else
        x(ind)=(-1);
    end
end
