% Compute testing and training error given a IRP path of isotonic solutions
% NOTE: Breaks the path into pieces with length <= numBreak in order to use
%       less memory per loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:  
% yTrain:       n1 x 1 vector of training observations
% yTest:        n2 x 1 vector of testing observations
% dataTrain:    m1 x n data matrix of training data, m1 samples and n features
% dataTest:     m2 x n data matrix of testing data, m2 samples and n features
% A:            n1 x n1 matrix s.t. A(i,j)==1 if y(i)<=y(j) and A(i,j)==0 otherwise
% cutInds:      structure of indices that represents the IRP path (returned by function IRP)
% numBreak:     maximum number of models in each segment of the path that
%                  is passed to the prediction functions (in order to use less memory)
% options:      same options as to IRP(.) function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Outputs:  
% errorTest:    overall error on testing data 
% errorTrain:   overall error on training data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modified by Ronny Luss, May 2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [errorTest,errorTrain] = predictWrapper(yTrain,yTest,dataTrain,dataTest,A,cutInds,numBreak,options)
if ~isfield(options,'weightedL2'), options.weightedL2=0; end

% storage for output
errorTest=zeros(1,size(cutInds,1));   
errorTrain=zeros(1,size(cutInds,1));    

d=floor(size(cutInds,1)/numBreak); % number of loops to make (in order to use less memory in prediction functions)
estimator=IRP_estimator(yTrain,options,1:length(yTrain)); % compute the estimator of group with all observations

disp('Finding comparables for each data test point');tic;
[indsL,indsU,compare]=find_comparables(dataTrain,dataTest);
disp(['Done finding comparables in ',num2str(toc),' seconds.']);

% First compute d loops of predictions in order to use less memory
disp(['Breaking the IRP path into ',num2str(d+1),' pieces.']);
for i=1:d
    disp(['Recreating IRP Path ',num2str(i)]);tic;
    pathIRP=reCreatePathIRP(yTrain,cutInds,numBreak*(i-1)+1,numBreak,options);
    disp(['Done recreating IRP Path in ',num2str(toc),' seconds.']);
    
    disp(['Starting predictions for path segment ',num2str(i)]);tic;
    yPred=predictOutOfSample(pathIRP,A,indsL,indsU,compare,estimator);
    disp(['Done running predicting after ',num2str(toc),' seconds.']);

    [errorTest((numBreak*(i-1)+1):(numBreak*(i-1)+numBreak)),errorTrain((numBreak*(i-1)+1):(numBreak*(i-1)+numBreak))]=predictLossFunction(yTrain,yTest,pathIRP,yPred);    
end

% Compute remaining predictions.  if total number predictions <= numBreak, only this IF will be run
if mod(size(cutInds,1),numBreak)>0 % compute for the end of the path
    disp(['Recreating IRP Path ',num2str(d+1)]);tic;
    pathIRP=reCreatePathIRP(yTrain,cutInds,numBreak*d+1,mod(size(cutInds,1),numBreak),options);
    disp(['Done recreating IRP Path in ',num2str(toc),' seconds.']);
    
    disp(['Starting predictions for path segment ',num2str(d+1)]);tic;
    yPred=predictOutOfSample(pathIRP,A,indsL,indsU,compare,estimator);
    disp(['Done running predicting after ',num2str(toc),' seconds.']);
    
    [errorTest((numBreak*d+1):(numBreak*d+mod(size(cutInds,1),numBreak))),errorTrain((numBreak*d+1):(numBreak*d+mod(size(cutInds,1),numBreak)))]=predictLossFunction(yTrain,yTest,pathIRP,yPred);    
end